@@ -2,8 +2,9 @@
 Name               : callbacks.c
 Author             : Marvin Raaijmakers
 Description        : Handles the GUI callbacks
-Date of last change: 03-Feb-2007
-History            : 03-Feb-2007 - The action member of a KTKeySettings is now
+Date of last change: 28-Jun-2007
+History            : 28-Jun-2007 Fixed compiler warnings
+                     03-Feb-2007 - The action member of a KTKeySettings is now
                                    modified by using a function of
                                    key_settings.(c/h)
                                  - on_plugin_remove_button_clicked():
@@ -66,7 +67,6 @@ History            : 03-Feb-2007 - The action member of a KTKeySettings is now
 /* Points to the key settings of the currently
  * selected key or to NULL if no key was selected: */
 static KTKeySettings *selected_key = NULL;
-static char *current_command = NULL;
 
 static void handle_changed_special_action (GtkToggleButton *special_action_radiobutton);
 static void handle_changed_program (GtkEntry *program_entry, GtkToggleButton *program_radiobutton);
@@ -470,7 +470,7 @@ Description:
 				program_user = selected_key->action.program.command;
 				radiobutton = GTK_RADIO_BUTTON( lookup_widget(GTK_WIDGET(key_treeview), "program_radiobutton") );
 			}
-			else if (selected_key->action.type == KTActionTypePlugin)
+			else /* if (selected_key->action.type == KTActionTypePlugin)*/
 			{
 				plugin = find_plugin (selected_key->action.plugin.plugin_name, &plugin_list);
 				if (plugin_function_exists (selected_key->action.plugin.plugin_name,
@@ -512,7 +512,7 @@ Description:
 					                  selected_key->default_action.plugin.function_name);
 				}
 			}
-			else if (selected_key->default_action.type == KTActionTypeProgram)
+			else /* if (selected_key->default_action.type == KTActionTypeProgram) */
 			{
 				default_action = (char *)g_strdup_printf(
 						selected_key->default_action.program.command);
@@ -683,7 +683,6 @@ on_plugin_combo_entry_changed          (GtkEditable     *editable,
 {
 	char			*plugin_name;
 	KTPluginInfoEntry	*plugin;
-	GtkWidget		*special_action_radiobutton;
 	
 	plugin_name = (char *) gtk_entry_get_text(GTK_ENTRY(editable));
 	plugin = find_plugin (plugin_name, &plugin_list);
@@ -715,4 +714,4 @@ on_special_action_radiobutton_toggled  (GtkToggleButton *togglebutton,
 	{
 		handle_changed_special_action (togglebutton);
 	}
-}
\ No newline at end of file
+}
