@@ -2,11 +2,12 @@
 Name               : check_syntax.c
 Author             : Marvin Raaijmakers
 Description        : Checks if the syntax of a keyboard file is valid
-Date of last change: 17-Sep-2006
-History            : 17-Sep-2006 Added support for ACPI hotkeys
+Date of last change: 28-Jun-2007
+History            : 28-Jun-2007 Fixed compiler warnings
+                     17-Sep-2006 Added support for ACPI hotkeys
                      24-Jun-2006 Added checks for keycodes and scancodes
 
-    Copyright (C) 2006 Marvin Raaijmakers
+    Copyright (C) 2006-2007 Marvin Raaijmakers
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -27,6 +28,8 @@ History            : 17-Sep-2006 Added support for ACPI hotkeys
 -----------------------------------------------------------------------------------*/
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
+#define _XOPEN_SOURCE /* glibc2 needs this */
 #include <time.h>
 
 #include <mxml.h>
