@@ -2,8 +2,12 @@
 Name               : config.c
 Author             : Marvin Raaijmakers
 Description        : Reads and writes the configuration files
-Date of last change: 30-Aug-2006
-History            : 30-Aug-2006 In read_key_settings strcmp, for comparing the
+Date of last change: 15-Oct-2006
+History            : 15-Oct-2006 The constants in config_element_type.h are now
+                                 used and reading/writing the "documents
+                                 directory" from/to the preferences file is
+                                 replaced by reading/writing the "mixer channel"
+                     30-Aug-2006 In read_key_settings strcmp, for comparing the
                                  keycode with "VOLUMEUP" or "VOLUMEDOWN", by
                                  strcasecmp
                      19-Mar-2006 get_current_keyboard() exists quietly if
@@ -42,6 +46,7 @@ History            : 30-Aug-2006 In read_key_settings strcmp, for comparing the
 
 #include <string_to_keycode.h>
 #include <keytouchd.h>
+#include <config_element_types.h>
 
 static KTKeySettings *read_key_settings (	XmlContent		*keylist_element,
 						int			index,
@@ -182,14 +187,14 @@ Desciption:
 		exit (EXIT_FAILURE);
 	}
 	
-	keyboard_type = XmlGetElementType("keyboard", document, FALSE);
+	keyboard_type = XmlGetElementType(ELEM_TYPE_KEYBOARD, document, FALSE);
 	if (keyboard_type == NULL ||
 	    (keyboard_element = XmlGetElementOfType(XmlDocumentContentList(document), keyboard_type, 0)) == NULL)
 	{
 		KTError ("The keyboard file '%s' does not contain a keyboard element.", file_name);
 		exit (EXIT_FAILURE);
 	}
-	keylist_type = XmlGetElementType("key-list", document, FALSE);
+	keylist_type = XmlGetElementType(ELEM_TYPE_KEYLIST, document, FALSE);
 	if (keylist_type == NULL ||
 	    (keylist_element =
 	       XmlGetElementOfType(XmlElementContentList(keyboard_element), keylist_type, 0)) == NULL)
@@ -198,9 +203,9 @@ Desciption:
 		exit (EXIT_FAILURE);
 	}
 	
-	if ((keycode_type = XmlGetElementType("keycode", document, FALSE)) == NULL ||
-	    (key_type = XmlGetElementType("key", document, FALSE)) == NULL ||
-	    (keyname_type = XmlGetElementType("name", document, FALSE)) == NULL)
+	if ((keycode_type = XmlGetElementType(ELEM_TYPE_KEYCODE, document, FALSE)) == NULL ||
+	    (key_type = XmlGetElementType(ELEM_TYPE_KEY, document, FALSE)) == NULL ||
+	    (keyname_type = XmlGetElementType(ELEM_TYPE_KEYNAME, document, FALSE)) == NULL)
 	{
 		KTError ("The keyboard file '%s' is invalid.", file_name);
 		exit (EXIT_FAILURE);
@@ -293,13 +298,13 @@ Desciption:
 		/* Get the type of the action */
 		action_type_argument = XmlGetAttributeValue(action_type_attrname, action_element);
 		/* If it is a program */
-		if (action_type_argument == NULL || strcmp(action_type_argument, "program") == EQUAL)
+		if (action_type_argument == NULL || strcmp(action_type_argument, ATTR_VAL_PROGRAM) == EQUAL)
 		{
 			key_settings->action.type = KTActionTypeProgram;
 			key_settings->action.program.command = XmlGetElementString(action_element, "");
 		}
 		/* If it is a plugin */
-		else if (strcmp(action_type_argument, "plugin") == EQUAL)
+		else if (strcmp(action_type_argument, ATTR_VAL_PLUGIN) == EQUAL)
 		{
 			key_settings->action.type = KTActionTypePlugin;
 			if (!(pluginname_element =
@@ -400,14 +405,14 @@ Desciption:
 		exit (EXIT_FAILURE);
 	}
 	
-	keyboard_type = XmlGetElementType("keyboard", document, FALSE);
+	keyboard_type = XmlGetElementType(ELEM_TYPE_KEYBOARD, document, FALSE);
 	if (keyboard_type == NULL ||
 	    (keyboard_element = XmlGetElementOfType(XmlDocumentContentList(document), keyboard_type, 0)) == NULL)
 	{
 		KTError ("The keyboard configuration file does not contain a keyboard element.", "");
 		exit (EXIT_FAILURE);
 	}
-	keylist_type = XmlGetElementType("key-list", document, FALSE);
+	keylist_type = XmlGetElementType(ELEM_TYPE_KEYLIST, document, FALSE);
 	if (keylist_type == NULL ||
 	    (keylist_element =
 	       XmlGetElementOfType(XmlElementContentList(keyboard_element), keylist_type, 0)) == NULL)
@@ -416,23 +421,23 @@ Desciption:
 		exit (EXIT_FAILURE);
 	}
 	
-	if ((action_type = XmlGetElementType("action", document, FALSE)) == NULL ||
-	    (key_type = XmlGetElementType("key", document, FALSE)) == NULL ||
-	    (keyname_type = XmlGetElementType("name", document, FALSE)) == NULL)
+	if ((action_type = XmlGetElementType(ELEM_TYPE_ACTION, document, FALSE)) == NULL ||
+	    (key_type = XmlGetElementType(ELEM_TYPE_KEY, document, FALSE)) == NULL ||
+	    (keyname_type = XmlGetElementType(ELEM_TYPE_KEYNAME, document, FALSE)) == NULL)
 	{
 		KTError ("The keyboard configuration file is invalid.", "");
 		exit (EXIT_FAILURE);
 	}
-	action_type_attrname = XmlGetAttributeName("action-type", document, TRUE);
+	action_type_attrname = XmlGetAttributeName(ATTR_NAME_ACTIONTYPE, document, TRUE);
 	/* Create the element types "plugin-name" and "plugin-function" if they
 	 * do not appear in the document, because these element do not necessary
 	 * appear in the document */
-	pluginname_type = XmlGetElementType("plugin-name", document, TRUE);
-	pluginfunction_type = XmlGetElementType("plugin-function", document, TRUE);
+	pluginname_type = XmlGetElementType(ELEM_TYPE_PLUGINNAME, document, TRUE);
+	pluginfunction_type = XmlGetElementType(ELEM_TYPE_PLUGINFUNCTION, document, TRUE);
 	
 	/* Read the action members of the keys */
 	for (count = 0;
-	     read_key_settings_action (key_list, keylist_element, count, key_type,                               
+	     read_key_settings_action (key_list, keylist_element, count, key_type,
 	                               keyname_type, action_type, action_type_attrname,
 				       pluginname_type, pluginfunction_type);
 	     count++)
@@ -501,14 +506,14 @@ Desciption:
 	XmlDocument	*document;
 	char		*file_name;
 	XmlElementType	*preferences_type,
-			*documents_dir_type,
+			*mixer_channel_type,
 			*home_page_type,
 			*search_page_type,
 			*browser_type,
 			*email_program_type,
 			*chat_program_type;
 	XmlContent	*preferences_element,
-			*documents_dir_element,
+			*mixer_channel_element,
 			*home_page_element,
 			*search_page_element,
 			*browser_element,
@@ -522,13 +527,13 @@ Desciption:
 		KTError ("'%s' is an invalid preferences configuration file.", file_name);
 		exit (EXIT_FAILURE);
 	}
-	if ((preferences_type = XmlGetElementType("preferences", document, FALSE)) == NULL ||
-	    (documents_dir_type = XmlGetElementType("documents-dir", document, FALSE)) == NULL ||
-	    (home_page_type = XmlGetElementType("home-page", document, FALSE)) == NULL ||
-	    (search_page_type = XmlGetElementType("search-page", document, FALSE)) == NULL ||
-	    (browser_type = XmlGetElementType("browser", document, FALSE)) == NULL ||
-	    (email_program_type = XmlGetElementType("email-program", document, FALSE)) == NULL ||
-	    (chat_program_type = XmlGetElementType("chat-program", document, FALSE)) == NULL)
+	if ((preferences_type = XmlGetElementType(ELEM_TYPE_PREFERENCES, document, FALSE)) == NULL ||
+	    (mixer_channel_type = XmlGetElementType(ELEM_TYPE_MIXERCHANNEL, document, FALSE)) == NULL ||
+	    (home_page_type = XmlGetElementType(ELEM_TYPE_HOMEPAGE, document, FALSE)) == NULL ||
+	    (search_page_type = XmlGetElementType(ELEM_TYPE_SEARCHPAGE, document, FALSE)) == NULL ||
+	    (browser_type = XmlGetElementType(ELEM_TYPE_BROWSER, document, FALSE)) == NULL ||
+	    (email_program_type = XmlGetElementType(ELEM_TYPE_EMAILPROG, document, FALSE)) == NULL ||
+	    (chat_program_type = XmlGetElementType(ELEM_TYPE_CHATPROG, document, FALSE)) == NULL)
 	{
 		KTError ("'%s' is an invalid preferences configuration file.", file_name);
 		exit (EXIT_FAILURE);
@@ -539,8 +544,8 @@ Desciption:
 		KTError ("'%s' does has not a preferences element as the root element.", file_name);
 		exit (EXIT_FAILURE);
 	}
-	if ((documents_dir_element =
-	      XmlGetElementOfType(XmlElementContentList(preferences_element), documents_dir_type, 0)) == NULL ||
+	if ((mixer_channel_element =
+	      XmlGetElementOfType(XmlElementContentList(preferences_element), mixer_channel_type, 0)) == NULL ||
 	    (home_page_element =
 	      XmlGetElementOfType(XmlElementContentList(preferences_element), home_page_type, 0)) == NULL ||
 	    (search_page_element =
@@ -555,7 +560,7 @@ Desciption:
 		KTError ("'%s' is an invalid preferences configuration file.", file_name);
 		exit (EXIT_FAILURE);
 	}
-	preferences->documents_dir = XmlGetElementString(documents_dir_element, "");
+	preferences->mixer_channel = XmlGetElementString(mixer_channel_element, "");
 	preferences->home_page = XmlGetElementString(home_page_element, "");
 	preferences->search_page = XmlGetElementString(search_page_element, "");
 	preferences->browser = XmlGetElementString(browser_element, "");
@@ -601,9 +606,9 @@ Desciption:
 		KTError ("'%s' is invalid", CURRENT_KEYBOARD_FILE);
 		exit (EXIT_FAILURE);
 	}
-	if ((current_keyboard_type = XmlGetElementType("current-keyboard", document, FALSE)) == NULL ||
-	    (model_type = XmlGetElementType("model", document, FALSE)) == NULL ||
-	    (manufacturer_type = XmlGetElementType("manufacturer", document, FALSE)) == NULL)
+	if ((current_keyboard_type = XmlGetElementType(ELEM_TYPE_CURRENTKEYBOARD, document, FALSE)) == NULL ||
+	    (model_type = XmlGetElementType(ELEM_TYPE_MODEL, document, FALSE)) == NULL ||
+	    (manufacturer_type = XmlGetElementType(ELEM_TYPE_MANUFACTURER, document, FALSE)) == NULL)
 	{
 		KTError ("'%s' is invalid", CURRENT_KEYBOARD_FILE);
 		exit (EXIT_FAILURE);
