@@ -1,9 +1,15 @@
 /*---------------------------------------------------------------------------------
 Name               : amixer.c
 Author             : Marvin Raaijmakers
-Description        : Plugin for keyTouch that can change the volume (using amixer).
-Date of last change: 24-Sep-2006
-History            : 24-Sep-2006 Added two new plugin functions:
+Description        : Plugin for keyTouch that can change the volume (via ALSA).
+Date of last change: 09-Dec-2006
+History            : 09-Dec-2006 Muting is now also possible for channels without
+                                 a playback switch
+                     19-Oct-2006 This plugin now uses the ALSA library for setting
+                                 and retrieving the volume, instead of a pipe to
+                                 amixer. It now also uses the mixer_channel element
+                                 of the KTPreferences structure.
+                     24-Sep-2006 Added two new plugin functions:
                                  "Volume increase 10%" and "Volume decrease 10%"
                      05-Mar-2006 - clean_exit() will be used to exit the client
                                    process, that manages the volume bar, cleanly
@@ -11,7 +17,7 @@ History            : 24-Sep-2006 Added two new plugin functions:
                                    if the function should be called again
                      29-Jan-2006 Added the GUI volume bar to the plugin
 
-    Copyright (C) 2005-2006 Marvin Raaijmakers
+    Copyright (C) 2005-2007 Marvin Raaijmakers
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -38,11 +44,13 @@ History            : 24-Sep-2006 Added two new plugin functions:
 #include <sys/ipc.h>
 #include <sys/msg.h>
 #include <string.h>
+#include <errno.h>
+#include <alsa/asoundlib.h>
+#include <math.h>
 
 #include <plugin.h>
 #include <amixer-plugin.h>
 
-
 void vol_increase (KTPreferences *preferences);
 void vol_decrease (KTPreferences *preferences);
 void vol_increase_10 (KTPreferences *preferences);
@@ -50,23 +58,20 @@ void vol_decrease_10 (KTPreferences *preferences);
 void mute (KTPreferences *preferences);
 
 static void create_window (VOLUMEBAR_INFO *volumebar_info);
-static int get_current_volume (void);
-static void update_volume_bar (GtkWidget *volume_bar);
+static double get_current_volume (const char *mix_elem_name);
+static void update_volume_bar (GtkWidget *volume_bar, const char *mix_elem_name);
 static gboolean update_window (VOLUMEBAR_INFO *volumebar_info);
 static void clean_exit (int sig);
-static void start_window (void);
+static void start_window (const char *mix_elem_name);
 static char *get_keytouch_user_dir (void);
-static void change_volume (char *command);
-
-
-static Boolean is_muted = FALSE;
+static void change_volume (int vol_incr, const char *mix_elem_name);
+static Boolean volume_is_muted (const char *mix_elem_name);
 
 
 KeytouchPlugin plugin_struct = {
-	{"Amixer", "Marvin Raaijmakers", "GPL 2", "2.3",
+	{"Amixer", "Marvin Raaijmakers", "GPL 2", "3.0",
 	 "This plugin allows you to change the volume. It also shows\n"
-	 "the current volume when it changes. To use this plugin amixer\n"
-	 "needs to be installed."},
+	 "the current volume when it changes."},
 	"amixer.so",
 	5,
 	{{"Volume increase",     KTPluginFunctionType_Function, {.function = vol_increase}},
@@ -78,6 +83,78 @@ KeytouchPlugin plugin_struct = {
 };
 
 
+/******* Begin ALSA part *******/
+
+snd_mixer_elem_t
+*get_mixer_elem (const char *control_name)
+/*
+Input:
+	control_name	- The name of the mixer element to get.
+Returns:
+	A pointer to the mixer element that is named 'control_name' or NULL if no
+	such element exists. IMPORTANT: This pointer will be valid until this
+	function is being called again (because then the related resources will be
+	freed).
+*/
+{
+	static snd_mixer_t *handle = NULL;
+	static snd_mixer_selem_id_t *sid;
+	snd_mixer_elem_t *elem;
+	int err;
+	
+	if (handle != NULL)
+	{
+		snd_mixer_close (handle);
+		handle = NULL;
+	}
+	snd_mixer_selem_id_alloca (&sid);
+	snd_mixer_selem_id_set_index (sid, 0);
+	snd_mixer_selem_id_set_name (sid, control_name);
+	
+	
+	if ((err = snd_mixer_open(&handle, 0)) < 0)
+	{
+		fprintf (stderr, "Mixer %s open error: %s\n", "default", snd_strerror(err));
+		return NULL;
+	}
+	if ((err = snd_mixer_attach(handle, "default")) < 0)
+	{
+		fprintf (stderr, "Mixer attach %s error: %s", "default", snd_strerror(err));
+		snd_mixer_close (handle);
+		handle = NULL;
+		return NULL;
+	}
+	if ((err = snd_mixer_selem_register(handle, NULL, NULL)) < 0)
+	{
+		fprintf (stderr, "Mixer register error: %s", snd_strerror(err));
+		snd_mixer_close (handle);
+		handle = NULL;
+		return NULL;
+	}
+	err = snd_mixer_load (handle);
+	if (err < 0)
+	{
+		fprintf (stderr, "Mixer %s load error: %s", "default", snd_strerror(err));
+		snd_mixer_close (handle);
+		handle = NULL;
+		return NULL;
+	}
+	
+	
+	elem = snd_mixer_find_selem (handle, sid);
+	if (!elem)
+	{
+		fprintf (stderr, "Unable to find simple control '%s',%i\n", snd_mixer_selem_id_get_name(sid), snd_mixer_selem_id_get_index(sid));
+		snd_mixer_close (handle);
+		handle = NULL;
+		return NULL;
+	}
+	
+	return elem;
+}
+
+/******* End ALSA part *******/
+
 
 void
 create_window (VOLUMEBAR_INFO *volumebar_info)
@@ -118,71 +195,101 @@ Description:
 	volumebar_info->volume_bar = gtk_progress_bar_new();
 	gtk_widget_show (volumebar_info->volume_bar);
 	gtk_container_add (GTK_CONTAINER (volumebar_info->window), volumebar_info->volume_bar);
-	gtk_widget_set_size_request (volumebar_info->volume_bar, 231, 24);
+	gtk_widget_set_size_request (volumebar_info->volume_bar, VOLUME_WINDOW_WIDTH, VOLUME_WINDOW_HEIGHT);
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (volumebar_info->volume_bar), 0.52);
 	gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (volumebar_info->volume_bar), 0.02);
 	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (volumebar_info->volume_bar), "Volume");
 }
 
 
-int
-get_current_volume (void)
+double
+get_current_volume (const char *mix_elem_name)
 /*
+Input:
+	mix_elem_name	- The name of the mixer element that should be used for
+			  retrieving the volume.
 Returns:
-	The current volume retrieved from amixer. -1 will be returned when
-	retrieving the volume failed.
+	The current volume level (0.00 = lowest to 1.00 = highest) retrieved from
+	amixer. -1.0 will be returned when retrieving the volume failed.
 */
 {
-	FILE *amixer;
-	char c;
-	int volume = -1;
+	snd_mixer_elem_t *mix_elem;
+	long vol_min, vol_max, cur_vol;
+	double volume = -1.0;
 	
-	amixer = popen ("amixer sget Master | grep \"Front Left:\"", "r");
-	if (amixer)
+	mix_elem = get_mixer_elem (mix_elem_name);
+	if (mix_elem)
 	{
-		do {
-			c = getc(amixer);
-			/* We have found the volume when the following appears:
-			 * '[' followed by an integer followed by '%'
-			 */
-			if (c == '[' && fscanf(amixer, "%d", &volume) && (c = getc(amixer)) == '%')
-			{
-				break;
-			}
-			volume = -1;
-		} while (c != '\n' && c != EOF);
-		pclose (amixer);
+		snd_mixer_selem_get_playback_volume_range (mix_elem, &vol_min, &vol_max);
+		snd_mixer_selem_get_playback_volume (mix_elem, 0, &cur_vol); /* Take the first channel (= 0) */
+		volume = (double)(cur_vol - vol_min) / ((double)(vol_max - vol_min));
 	}
 	return (volume);
 }
 
 
-void
-update_volume_bar (GtkWidget *volume_bar)
+Boolean
+volume_is_muted (const char *mix_elem_name)
 /*
-Output:
-	volume_bar	- Will show the percentage of the current volume
+Input:
+	mix_elem_name	- The name of the mixer element that should be used for
+			  retrieving the state of the mute switch.
+Returns:
+	TRUE if the first channel (= 0) of the mixer element named 'mix_elem_name'
+	is muted, otherwise FALSE.
 */
 {
-	int volume;
+	snd_mixer_elem_t *mix_elem;
+	int not_muted = TRUE;
+	
+	mix_elem = get_mixer_elem (mix_elem_name);
+	if (mix_elem)
+	{
+		snd_mixer_selem_get_playback_switch (mix_elem, 0, &not_muted); /* Take the first channel (= 0) */
+	}
+	
+	return (Boolean) !not_muted;
+}
+
+
+void
+update_volume_bar ( GtkWidget  *volume_bar,
+                    const char *mix_elem_name )
+/*
+Input:
+	mix_elem_name	- The name of the mixer element that should be used for
+			  retrieving the volume.
+Output:
+	volume_bar	- Will show the percentage of the current volume. The text
+			  on this progressbar will be equal to "Volume #%" where #
+			  is the volume percentage, if the volume is not muted. If
+			  the volume is muted then the text will be "Muted".
+*/
+{
+	double volume;
 	gchar *text;
 	
-	volume = get_current_volume();
-	if (volume && volume != -1)
+	volume = get_current_volume (mix_elem_name);
+	if (volume < .00)
 	{
-		text = g_strdup_printf("Volume %d%%", volume);
+		volume = .00;
+	}
+	
+	if (volume_is_muted (mix_elem_name))
+	{
+		gtk_progress_bar_set_text (GTK_PROGRESS_BAR(volume_bar), "Muted");
+	}
+	else
+	{
+		text = g_strdup_printf("Volume %d%%", (int)(volume*100));
 		if (text)
 		{
 			gtk_progress_bar_set_text (GTK_PROGRESS_BAR(volume_bar), text);
 			g_free (text);
 		}
 	}
-	else
-	{
-		volume = 0;
-		gtk_progress_bar_set_text (GTK_PROGRESS_BAR(volume_bar), "Muted");
-	}
-	gtk_progress_set_percentage (GTK_PROGRESS(volume_bar), (gdouble)volume/100.0);
+	
+	gtk_progress_set_percentage (GTK_PROGRESS(volume_bar), (gdouble)volume);
 	/* Directly draw the progressbar: */
 	while (g_main_context_iteration(NULL, FALSE))
 		; /* NULL Statement */
@@ -191,9 +298,12 @@ Output:
 
 gboolean
 update_window (VOLUMEBAR_INFO *volumebar_info)
+
 /*
 Input:
 	volumebar_info->close_time	- The time to close the window
+	volumebar_info->mix_elem_name	- The name of the mixer element that should
+					  be used for retrieving the volume.
 Output:
 	volumebar_info			- Will be updated
 Returns:
@@ -216,7 +326,7 @@ Description:
 	close_window = (time(NULL) > volumebar_info->close_time);
 	if (!close_window)
 	{
-		update_volume_bar (volumebar_info->volume_bar);
+		update_volume_bar (volumebar_info->volume_bar, volumebar_info->mix_elem_name);
 	}
 	else
 	{
@@ -228,8 +338,11 @@ Description:
 
 
 void
-start_window (void)
+start_window (const char *mix_elem_name)
 /*
+Input:
+	mix_elem_name	- The name of the mixer element that should be used for
+			  retrieving the volume.
 Description:
 	This function creates a window with a volume bar and shows it
 	SHOW_WINDOW_TIME seconds when it receives a message on the message queue.
@@ -256,12 +369,15 @@ Description:
 		return;
 	}
 	/* Get the message queue identifier and create the queue if necessary */
-	volumebar_info.msgqid = msgget(msgq_key, 0);
+	volumebar_info.msgqid = msgget (msgq_key, 0);
 	if (volumebar_info.msgqid == -1)
 	{
 		perror ("keytouch amixer plugin");
 		return;
 	}
+	
+	volumebar_info.mix_elem_name = mix_elem_name;
+	
 	while (1)
 	{
 		if (msgrcv(volumebar_info.msgqid, &msg, sizeof(msg.time), 1, 0) != -1)
@@ -270,7 +386,7 @@ Description:
 			if (time(NULL) <= volumebar_info.close_time)
 			{
 				create_window (&volumebar_info);
-				update_volume_bar (volumebar_info.volume_bar);
+				update_volume_bar (volumebar_info.volume_bar, mix_elem_name);
 				gtk_widget_show (volumebar_info.window);
 				g_timeout_add (UPDATE_INTERVAL, (GSourceFunc) update_window, &volumebar_info);
 				gtk_main();
@@ -314,12 +430,18 @@ clean_exit (int sig)
 
 
 void
-send_volume_changed_signal (void)
+send_volume_changed_signal (const char *mix_elem_name)
 /*
+Input:
+	mix_elem_name	- The name of the mixer element that should be used for
+			  retrieving the volume.
 Description:
 	This function sends a signal to the child program that manages the
 	volumebar. The child will receive the signal and will show the volumebar.
-	The child process will be created if it does not exist yet.
+	The child process will be created if it does not exist yet. The volume of
+	the mixer element named 'mix_elem_name' will be used. When this function is
+	called for the first time, then the value of 'mix_elem_name' will also be
+	used for the next time this function is called.
 */
 {
 	static int qid = -1;
@@ -356,9 +478,9 @@ Description:
 			/* We will now start the run_window() function in our
 			 * child process for showing a volume bar to the user
 			 */
-			start_window();
+			start_window (mix_elem_name);
 			exit (EXIT_SUCCESS); /* We will never get here because of
-			                      * the infinite loop in run_window()
+			                      * the infinite loop in start_window()
 			                      */
 		}
 	}
@@ -372,23 +494,50 @@ Description:
 
 
 void
-change_volume (char *command)
+change_volume (	int		vol_incr,
+		const char	*mix_elem_name )
 /*
 Input:
-	command	- The command that changes the volume.
+	command		- The command that changes the volume.
+	mix_elem_name	- The name of the mixer element to use.
 Description:
 	This function executes 'command' in a child process and then calls
 	send_volume_changed_signal().
 */
 {
-	if (fork() == 0)
+	long vol_min, vol_max, cur_vol;
+	snd_mixer_selem_channel_id_t chn;
+	snd_mixer_elem_t *mix_elem;
+	
+	mix_elem = get_mixer_elem (mix_elem_name);
+	if (mix_elem)
 	{
-		execlp ("sh", "sh", "-c", command, NULL);
-		exit (EXIT_SUCCESS);
-	}
-	else
-	{
-		send_volume_changed_signal();
+		snd_mixer_selem_get_playback_volume_range (mix_elem, &vol_min, &vol_max);
+		
+		for (chn = 0; chn <= SND_MIXER_SCHN_LAST; chn++)
+		{
+			if (snd_mixer_selem_has_playback_channel(mix_elem, chn))
+			{
+				snd_mixer_selem_get_playback_volume (mix_elem, chn, &cur_vol);
+				cur_vol += rint((double)(vol_max - vol_min) * ((double)vol_incr*.01));
+				if (cur_vol > vol_max)
+				{
+					cur_vol = vol_max;
+				}
+				else if (cur_vol < vol_min)
+				{
+					cur_vol = vol_min;
+				}
+				snd_mixer_selem_set_playback_volume (mix_elem, chn, cur_vol);
+				/* If the volume level of this channel also controls the level
+				 * of the other channels of 'mix_elem' */
+				if (snd_mixer_selem_has_playback_volume_joined (mix_elem))
+				{
+					break;
+				}
+			}
+		}
+		send_volume_changed_signal (mix_elem_name);
 	}
 }
 
@@ -396,76 +545,101 @@ Description:
 void
 vol_increase (KTPreferences *preferences)
 {
-	is_muted = FALSE;
-	change_volume ( CHANGE_VOL_CMD(VOL_DEFAULT_INCR) );
+	change_volume (VOL_DEFAULT_INCR, preferences->mixer_channel);
 }
 
 
 void
 vol_decrease (KTPreferences *preferences)
 {
-	is_muted &= !get_current_volume();
-	change_volume ( CHANGE_VOL_CMD(VOL_DEFAULT_DECR) );
+	change_volume (VOL_DEFAULT_DECR, preferences->mixer_channel);
 }
 
 
 void
 vol_increase_10 (KTPreferences *preferences)
 {
-	is_muted = FALSE;
-	change_volume ( CHANGE_VOL_CMD(VOL_10PERCENT_INCR) );
+	change_volume (VOL_10PERCENT_INCR, preferences->mixer_channel);
 }
 
 
 void
 vol_decrease_10 (KTPreferences *preferences)
 {
-	is_muted &= !get_current_volume();
-	change_volume ( CHANGE_VOL_CMD(VOL_10PERCENT_DECR) );
+	change_volume (VOL_10PERCENT_DECR, preferences->mixer_channel);
 }
 
 
 void
 mute (KTPreferences *preferences)
 {
-	static int	prev_volume = -1;
-	int		current_volume;
-	char		*command = NULL;
+	snd_mixer_elem_t *mix_elem;
+	int unmuted;
+	snd_mixer_selem_channel_id_t chn;
+	/* If no mixer channel does not have a playback switch (for muting), the value
+	 * of 'no_switch_muted' indicates whether the plugin has muted the volume (TRUE)
+	 * or not (FALSE) */
+	static Boolean no_switch_muted = FALSE;
+	static long prev_volume = -1;
+	long volume, vol_min, vol_max;
 	
-	current_volume = get_current_volume();
-	is_muted &= !current_volume;
-	if (is_muted)
+	mix_elem = get_mixer_elem (preferences->mixer_channel);
+	if (mix_elem)
 	{
-		/* Tell amixer to set the volume to prev_volume */
-		if (asprintf(&command, "amixer sset Master %d%% > /dev/null", prev_volume) == -1)
+		/* if mix_elem has a playback switch for muting: */
+		if (snd_mixer_selem_has_playback_switch(mix_elem))
 		{
-			fputs ("keytouch amixer plugin: asprintf() failed. "
-			       "This is probably caused because it failed to allocate memory.", stderr);
-		}
-	}
-	else if (current_volume)
-	{
-		/* Tell amixer to set the volume to 0 */
-		command = strdup("amixer sset Master 0% > /dev/null");
-		if (command == NULL)
-		{
-			perror ("keytouch amixer plugin");
-		}
-		prev_volume = current_volume;
-	}
-	/* Do we have to mute/unmute? */
-	if (command)
-	{
-		if (fork() == 0)
-		{
-			execlp ("sh", "sh", "-c", command, NULL);
-			exit (EXIT_SUCCESS);
+			/* Toggle the switch for every channel: */
+			for (chn = 0; chn <= SND_MIXER_SCHN_LAST; chn++)
+			{
+				/* Is it possible to mute/unmute this channel? */
+				if (snd_mixer_selem_has_playback_channel (mix_elem, chn))
+				{
+					snd_mixer_selem_get_playback_switch (mix_elem, chn, &unmuted);
+					snd_mixer_selem_set_playback_switch (mix_elem, chn, (unmuted ? 1 : 0) ^ 1);
+					/* If the mute switch of this channel also controls the mute switch
+					 * of the other channels of 'mix_elem' */
+					if (snd_mixer_selem_has_playback_switch_joined (mix_elem))
+					{
+						break;
+					}
+				}
+			}
 		}
 		else
 		{
-			send_volume_changed_signal();
+			snd_mixer_selem_get_playback_volume_range (mix_elem, &vol_min, &vol_max);
+			snd_mixer_selem_get_playback_volume (mix_elem, 0, &volume);
+			/* If the volume level is not equal to the lowest possible level then
+			 * the mixer channel is not muted: */
+			no_switch_muted &= !(volume - vol_min);
+			if (no_switch_muted) /* if muted */
+			{
+				volume = prev_volume;
+			}
+			else
+			{
+				prev_volume = volume;
+				volume = vol_min;
+			}
+			no_switch_muted = !no_switch_muted;
+			
+			/* Set the new volume for every channel: */
+			for (chn = 0; chn <= SND_MIXER_SCHN_LAST; chn++)
+			{
+				/* Is it possible to mute/unmute this channel? */
+				if (snd_mixer_selem_has_playback_channel (mix_elem, chn))
+				{
+					snd_mixer_selem_set_playback_volume (mix_elem, chn, volume);
+					/* If the volume level of this channel also controls the level
+					 * of the other channels of 'mix_elem' */
+					if (snd_mixer_selem_has_playback_volume_joined (mix_elem))
+					{
+						break;
+					}
+				}
+			}
 		}
-		free (command);
-		is_muted = !is_muted;
+		send_volume_changed_signal (preferences->mixer_channel);
 	}
 }
